﻿SuperStrict

Import "gfx/blitz.o"

' Imports
Framework brl.basic
Import sidesign.minib3d
Import brl.map
Import brl.DirectSoundAudio
Import brl.freetypefont
Import brl.tgaloader

Import brl.timer
Import brl.pngloader
Import brl.jpgloader
Import brl.EventQueue
Import pub.FreeProcess
Import maxgui.maxgui
Import maxgui.Drivers

' Constants
Const MODE_FULLSCREEN:Int = 1
Const MODE_WINDOW:Int = 2
Const SEGMENTS:Int = 360

Const PLANETS:Int = 10

' Globals
Global SYSTEM_SCREEN_WIDTH:Int = 1024
Global SYSTEM_SCREEN_HEIGHT:Int = 512
Global SYSTEM_SCREEN_DEPTH:Int = 32
Global SYSTEM_SCREEN_MODE:Int = MODE_WINDOW
Global SYSTEM_SCREEN_RATE:Int = 60
Global SYSTEM_SCREEN_SCALE:Float = SYSTEM_SCREEN_WIDTH * 1.0 / 2560
Global INI_LogToFile:Int = False
Global INI_LogToConsole:Int = True

Global FLAG_LINES:Int = 1
Global FLAG_NAMES:Int = 1

Global si:Int

Global YESNO:String[] = ["NO", "YES"]

Global systemid:Int = 0

Global Template:String
Global swidth:Int
Global sheight:Int
Global sfull:Int

Include "inc/Settings.bmx"

SeedRnd systemid

' ----------------------------------------------------------------------------
' 3D Graphics starts here
' ----------------------------------------------------------------------------
Graphics3D (SYSTEM_SCREEN_WIDTH, SYSTEM_SCREEN_HEIGHT, SYSTEM_SCREEN_DEPTH, 2 - SYSTEM_SCREEN_MODE, SYSTEM_SCREEN_RATE)
SetIcon(AppFile, GetActiveWindow())

' ------------------------------------------------------------------------------------------------
' Init Application
' ------------------------------------------------------------------------------------------------


' Functions
Include "inc/Math.bmx"
Include "inc/Mesh.bmx"
Include "inc/Mouse.bmx"
Include "inc/Pixel.bmx"
Include "inc/Text.bmx"
Include "inc/Starname.bmx"
Include "inc/System.bmx"
Include "inc/Win32.bmx"

' Types
Include "inc/TCam.bmx"
Include "inc/TFlag.bmx"
Include "inc/TMedia.bmx"
Include "inc/TMilkyway.bmx"
Include "inc/TQuad.bmx"
Include "inc/TSimplex.bmx"
Include "inc/TSystem.bmx"
Include "inc/TPlanet.bmx"

' Modules
Include "mod/mod.rover.bmx"

' Types
Global CAMERA:TCam = New TCam
Global MEDIA:TMedia = New TMedia
Global MILKYWAY:TMilkyway = New TMilkyway

' Meshes
Global FroZone:TMesh
Global HabZone:TMesh
Global HotZone:TMesh
Global OptZone:TMesh
Global EarthZone:TMesh

Global Star:TMesh
Global StarSprite:TSprite
Global StarGlow1:TSprite
Global StarGlow2:TSprite

' Log
'LogInit()

' Camera
CAMERA.add()
CAMERA.speed = 0.0001
CAMERA.RangeMin = 0.0001
CAMERA.RangeMax = 100000
CAMERA.y = 0.25
CAMERA.z = -1
CAMERA.Updateposition

' Initialize
LoadMedia()
LoadMilkyway()
InitSystem()

Global System:TSystem
NewSystem(systemid)

' position camera
PositionEntity CAMERA.cam, 0, 0, 0
RotateEntity CAMERA.cam, -15, 0, 0
MoveEntity CAMERA.cam, 0, 0, 100
PointEntity CAMERA.cam, Star
CenterMouse()

' ------------------------------------------------------------------------------------------------
' MAIN LOOP
' ------------------------------------------------------------------------------------------------
Repeat

	If AppTerminate() Then End
		
	If KeyHit(KEY_ESCAPE) Then End
	
	If MouseHit(1) Then NewSystem()
	
	si:+10
		
	' Camera
	CAMERA.zoom = 1
	If MouseDown(2) Then
	
		CAMERA.zoom = 25
		
	EndIf
	
	CAMERA.speed = 0.000001 * EntityDistance(CAMERA.cam, Star) * (1.0 + (KeyDown(KEY_SPACE) * 100))
	CAMERA.Movement
	CAMERA.Rotation
	CAMERA.Update
	MILKYWAY.keepPosition(CAMERA.cam)
	
	TurnEntity Star, 0, 0.1, 0
	
	HideEntity StarSprite
	HideEntity StarGlow1
	HideEntity StarGlow2
	
	If KeyHit(KEY_1) Then PlanetOutput(System, 1)
	If KeyHit(KEY_2) Then PlanetOutput(System, 2)
	If KeyHit(KEY_3) Then PlanetOutput(System, 3)
	If KeyHit(KEY_4) Then PlanetOutput(System, 4)
	If KeyHit(KEY_5) Then PlanetOutput(System, 5)
	If KeyHit(KEY_6) Then PlanetOutput(System, 6)
	If KeyHit(KEY_7) Then PlanetOutput(System, 7)
	If KeyHit(KEY_8) Then PlanetOutput(System, 8)
	If KeyHit(KEY_9) Then PlanetOutput(System, 9)
	If KeyHit(KEY_0) Then PlanetOutput(System, 10)
	
	If KeyHit(KEY_TAB) Then SystemOutput(System)
				
	RenderWorld
	
	' Draw Orbits	
	DrawOrbits()
	
	' 2D output
	BeginMax2D()
	
		' distance
		Local dist:Double = EntityDistance(Camera.cam, Star) ' dist = AU!!!
		Local lydist:Float = dist * System.LightYearAU
		Local watt:Float = CalcWatt(dist)
		Local temp:Float = CalcTemp(watt)
						
		' text output
		SetAlpha(1.0)
		SetBlend ALPHABLEND

		SetColor 0, 0, 0
		DrawText "Star Type:   " + System.SpectralClass, 1, 1
		DrawText "Distance:    " + Round(dist, 3) + " AU [" + Round(lydist, 4) + " LY]", 1, 16
		DrawText "Luminance:   " + Round(watt, 2) + " Watt/sqm", 1, 31
		DrawText "Temperature: " + Round(temp, 1) + "°C", 1, 46
		DrawText "System ID:   " + System.id, 1, 61
		
		SetColor 0, 255, 0
		DrawText "Star Type:   " + System.SpectralClass, 0, 0
		DrawText "Distance:    " + Round(dist, 3) + " AU [" + Round(lydist, 4) + " LY]", 0, 15
		DrawText "Luminance:   " + Round(watt, 2) + " Watt/sqm", 0, 30
		DrawText "Temperature: " + Round(temp, 1) + "°C", 0, 45
		DrawText "System ID:   " + System.id, 0, 60
					
		For Local i:Int = 1 To PLANETS
	
			Local P:TPlanet = System.Planet[i]
	
			Local d:Float = P.DistanceAU
			
			Local e:Int = Int(P.Evaluationscore)
			Local c:String = P.Category
			
			If c = "None" Then
			
				If P.TemperatureCategory Then c = P.TemperatureCategory + " " + P.SizeCategory Else c = P.SizeCategory
				
			EndIf
			
			Local temperature:Float = CalcTemp(CalcWatt(d))
			
			If temperature < 3000 Then
			
				If e >= 75 Then

					SetColor 0, 0, 0
					DrawText i + ": " + d + " " + c + " [" + Int(P.Evaluationscore) + "%, ESI: " + Round(P.ESI * 100, 2) + "] " + Round(temperature, 1) + "°C", 1, 71 + (i * 15)
					
					SetColor 0, 255, 0
					DrawText i + ": " + d + " " + c + " [" + Int(P.Evaluationscore) + "%, ESI: " + Round(P.ESI * 100, 2) + "] " + Round(temperature, 1) + "°C", 0, 70 + (i * 15)
					
				Else

					SetColor 0, 0, 0
					DrawText i + ": " + d + " " + c + " " + Round(temperature, 1) + "°C", 1, 71 + (i * 15)
				
					SetColor 255, 0, 0
					DrawText i + ": " + d + " " + c + " " + Round(temperature, 1) + "°C", 0, 70 + (i * 15)
				
				End If
				
			Else
			
				SetColor 0, 0, 0
				DrawText i + ": Vaporized (" + Round(temperature, 1) + "°C)", 1, 71 + (i * 15)

				SetColor 255, 0, 0
				DrawText i + ": Vaporized (" + Round(temperature, 1) + "°C)", 0, 70 + (i * 15)
				
			EndIf
		
		Next
		
	EndMax2D()
	
	Delay 1
	
	Flip
		
	GCCollect()
	
Forever

End


Function CalcWatt:Float(dist:Float)

	' temperature
	Local watt:Double
	Local t1:Double = (149.6 * 10 ^ 9) * dist
	watt = System.CorrectedHeatOutput * 1.0 / ((4.0 * Pi * t1 * t1))

	Return watt

End Function



Function CalcTemp:Float(watt:Float)

	Return (((watt / 4.0) / System.Boltzmann) ^ 0.25) - System.CelsiusKelvin

End Function



' ------------------------------------------------------------------------------------------------
' Creates Zone Meshes and central Star
' ------------------------------------------------------------------------------------------------
Function InitSystem()
	
	FroZone = CreateGlow(Null, 0, 0, SEGMENTS, 1 + 2 + 16 + 32, 1, 0, 64, 128, 1, 0, 64, 128, 0)
	EntityOrder FroZone, 600
	
	HabZone = CreateGlow(Null, 0, 0, SEGMENTS, 1 + 2 + 16 + 32, 1, 255, 255, 0, 1, 0, 255, 0, 0)
	EntityOrder HabZone, 500
	
	HotZone = CreateGlow(Null, 0, 0, SEGMENTS, 1 + 2 + 16 + 32, 1, 255, 0, 0, 1, 255, 255, 0, 0)
	EntityOrder HotZone, 400
	
	OptZone = CreateGlow(Null, 0, 0, SEGMENTS, 1 + 2 + 16 + 32, 1, 0, 255, 0, 0.25, 0, 255, 0, 0.25)
	EntityOrder OptZone, 300
	EntityBlend OptZone, 3
			
	Star = CreateSphere(64)
	EntityFX Star, 1
	EntityTexture Star, MEDIA.gett("STARSURF")
	TextureBlend MEDIA.gett("STARSURF"), 5
	
	StarSprite = CreateSprite(Star)
	EntityTexture StarSprite, MEDIA.gett("STARATMO")
	EntityOrder StarSprite, -1
	TextureBlend MEDIA.gett("STARATMO"), 5
	
	StarGlow1 = CreateSprite(Star)
	EntityTexture StarGlow1, MEDIA.gett("STARGLOW")
	EntityOrder StarGlow1, -10
	
	StarGlow2 = CreateSprite(Star)
	EntityTexture StarGlow2, MEDIA.gett("STARGLOW")
	EntityOrder StarGlow2, -100
	
	TextureBlend MEDIA.gett("STARGLOW"), 5
	
	EntityTexture OptZone, MEDIA.gett("GRADIENT")

End Function



' ------------------------------------------------------------------------------------------------
' Draw Orbits using OpenGL functions
' ------------------------------------------------------------------------------------------------
Function DrawOrbits()

	Local i:Int
	Local P:TPLanet
	Local r:Float

	'glEnable(GL_COLOR_MATERIAL)
	glLineWidth(1.0)
	glBegin(GL_LINES)
	
	For i = 1 To PLANETS
	
		P = System.Planet[i]
		r = P.DistanceAU
		
		Orbit(r, 255, 255, 255, 0.5)
				
	Next
	
	'glEnd()

	'glLineWidth(4.0)
	'glBegin(GL_LINES)

	' solar system comparison
	Orbit(40.0, 255, 0, 255, 1)
	
	' Optimal planet position
	Orbit(System.HZOptMinimumPlanetDistanceAU, 0, 255, 0, (1 + Sin(si)) / 8.0)
	glEnd()
	'glDisable(GL_COLOR_MATERIAL)

End Function


Function Orbit(s:Float, r:Int = 255, g:Int = 255, b:Int = 255, a:Float = 1.0)

	Local i:Int
	Local x1:Float, y1:Float, z1:Float
	Local x2:Float, y2:Float, z2:Float
	
	For i = 0 To 359
	
		x1 = Sin(i) * s
		y1 = 0
		z1 = Cos(i) * s
		x2 = Sin(i + 1) * s
		y2 = 0
		z2 = Cos(i + 1) * s
			
		Line3D(x1, y1, z1, x2, y2, z2, r, g, b, a)
			
	Next
		
End Function



' ------------------------------------------------------------------------------------------------
' Color Star according to spectral class
' ------------------------------------------------------------------------------------------------
Function StarColor()

	Local r:Int, g:Int, b:Int

	Select System.SpectralType
		
		Case "W" r = 255;g = 255;b = 255
		Case "O" r = 64;g = 128;b = 255
		Case "B" r = 128;g = 192;b = 255
		Case "A" r = 255;g = 255;b = 255
		Case "F" r = 255;g = 224;b = 192
		Case "G" r = 255;g = 224;b = 64
		Case "K" r = 255;g = 128;b = 64
		Case "M" r = 255;g = 32;b = 0
		
	End Select
	
	EntityColor Star, r, g, b
	EntityColor StarSprite, r, g, b
	EntityColor StarGlow1, r, g, b
	EntityColor StarGlow2, r, g, b
	
End Function



' ------------------------------------------------------------------------------------------------
' Load Media library
' ------------------------------------------------------------------------------------------------
Function LoadMedia()

	MEDIA.Init(CAMERA.cam)
	
	' Loader
	MEDIA.Load("T", "LOADER_TEXT", "textures/sprites/loader_text.tga", 1)
	MEDIA.Load("T", "LOADER_SPIRAL", "textures/sprites/loader_spiral.tga", 2 + 16 + 32)
	MEDIA.Loader_Text_Texture = MEDIA.GetT("LOADER_TEXT")
	MEDIA.Loader_Spiral_Texture = MEDIA.GetT("LOADER_SPIRAL")
		
	' Fonts
	MEDIA.Load("F", "STANDARD", "fonts/extrasolar.otf", 16)
	MEDIA.Load("F", "GUI", "fonts/interfacemono.otf", 16)
	MEDIA.Load("F", "GUISMALL", "fonts/interfacemono.otf", 8)
	MEDIA.Load("F", "LCD", "fonts/calculator.otf", 16)
	MEDIA.Load("F", "LARGE", "fonts/expolomono.otf", 32)
	MEDIA.Load("F", "GUI", "fonts/interfacemono.otf", 16)

	' Milkyway Textures
	MEDIA.Load("T", "MILKYWAY_STRIP", "textures/milkyway/milkyway_strip.jpg", 1 + 16 + 32)
	MEDIA.Load("T", "MILKYWAY_DETAIL", "textures/milkyway/milkyway_detail.jpg", 1)
	MEDIA.Load("T", "MILKYWAY_STARS", "textures/milkyway/milkyway_stars.png", 1)
	MEDIA.Load("T", "MILKYWAY_NEBULA", "textures/milkyway/milkyway_nebula.tga", 1 + 2 + 16 + 32)
	MEDIA.Load("T", "MILKYWAY_STARFIELD1", "textures/milkyway/milkyway_starfield1.png", 1 + 2)
	MEDIA.Load("T", "MILKYWAY_STARFIELD2", "textures/milkyway/milkyway_starfield2.png", 1 + 2)
	MEDIA.Load("T", "MILKYWAY_STARFIELD3", "textures/milkyway/milkyway_starfield3.png", 1 + 2)
	
	' Sprites
	MEDIA.Load("T", "STAR", "textures/sprites/sun.tga", 2)
	MEDIA.Load("T", "STARATMO", "textures/sprites/sun.png", 2)
	MEDIA.Load("T", "STARGLOW", "textures/sprites/glow.png", 2)
	MEDIA.Load("T", "ASTEROID", "textures/sprites/asteroid.png", 4)
	
	' Test
	MEDIA.Load("T", "STARSURF", "textures/starsurf.tga", 1 + 16 + 32)
	MEDIA.Load("T", "GRADIENT", "textures/gradfull.tga", 2 + 16 + 32)
	
			
End Function


' ------------------------------------------------------------------------------------------------
' Milkyway background
' ------------------------------------------------------------------------------------------------
Function LoadMilkyway()
	
	MILKYWAY.StarSphereTexture = MEDIA.GetT("STAR")

	MILKYWAY.PanoramaTexture = MEDIA.GetT("MILKYWAY_STRIP")
	MILKYWAY.DetailTexture = MEDIA.GetT("MILKYWAY_DETAIL")
	MILKYWAY.StarsTexture = MEDIA.GetT("MILKYWAY_STARS")
	MILKYWAY.NebulaTexture = MEDIA.GetT("MILKYWAY_NEBULA")
	MILKYWAY.StarBox1Texture = MEDIA.GetT("MILKYWAY_STARFIELD1")
	MILKYWAY.StarBox2Texture = MEDIA.GetT("MILKYWAY_STARFIELD2")
	MILKYWAY.StarBox3Texture = MEDIA.GetT("MILKYWAY_STARFIELD3")
	MILKYWAY.StarScale = 1.5 * SYSTEM_SCREEN_SCALE

	MILKYWAY.Seed = Rand(0, 2 ^ 16)
	MILKYWAY.Init

End Function



' ------------------------------------------------------------------------------------------------
' Create a Glow Mesh
' ------------------------------------------------------------------------------------------------
Function CreateGlow:TMesh(parent:TEntity = Null, radius1:Float = 1.0, radius2:Float = 2.0, segments:Int = 360, fx:Int = 0, blend:Int = 0, r1:Int = 255, g1:Int = 255, b1:Int = 255, al1:Float = 1.0, r2:Int = 255, g2:Int = 255, b2:Int = 255, al2:Float = 1.0, Inverse:Int = False)
	
	Local a1:Float,a2:Float,a3:Float,a4:Float,angle:Int
	Local v:Int,v0:Int,v1:Int,v2:Int,v3:Int
	Local mesh:TMesh=CreateMesh(parent)
	Local surf:TSurface=CreateSurface(mesh)
	
	If segments>360  Then segments=360
	
	For angle=1 To segments
		
		a1=angle*360.0/segments
		a2=angle*360.0/segments +360.0/segments
		a3=angle*360.0/segments +180.0/segments
		a4=angle*360.0/segments -180.0/segments
		
		If inverse Then

			v0=AddVertex(surf,radius1*Cos(a1),radius1*Sin(a1),0,1,0)
			v1=AddVertex(surf,radius1*Cos(a2),radius1*Sin(a2),0,1,1)
			v2=AddVertex(surf,radius2*Cos(a3),radius2*Sin(a3),0,0,0)
			v3=AddVertex(surf,radius2*Cos(a4),radius2*Sin(a4),0,0,1)

		Else

			v0 = AddVertex(surf, radius1 * Cos(a1), radius1 * Sin(a1), 0, 0, 0)
			v1 = AddVertex(surf, radius1 * Cos(a2), radius1 * Sin(a2), 0, 0, 0)
			v2 = AddVertex(surf, radius2 * Cos(a3), radius2 * Sin(a3), 0, 1, 1)
			v3 = AddVertex(surf, radius2 * Cos(a4), radius2 * Sin(a4), 0, 1, 1)
		
		EndIf

		VertexColor surf,v0,r1,g1,b1,al1
		VertexColor surf,v1,r1,g1,b1,al1
		VertexColor surf,v2,r2,g2,b2,al2
		VertexColor surf,v3,r2,g2,b2,al2
		
		AddTriangle surf,v2,v1,v0
		AddTriangle surf,v0,v3,v2
		
	Next
	
	For v=0 To CountVertices(surf)-1
		
		VertexTexCoords surf,v,-VertexX(surf,v),-VertexY(surf,v),0,1
		
	Next
	
	If fx>0 Then EntityFX mesh,fx
	If blend>0 Then EntityBlend mesh,blend
	
	Return mesh
	
End Function



' ------------------------------------------------------------------------------------------------
' Update a Glow Mesh
' ------------------------------------------------------------------------------------------------
Function UpdateGlow(mesh:TMesh, radius1:Float = 1.0, radius2:Float = 2.0, segments:Int = 360)
	
	Local an1:Float,an2:Float,an3:Float,an4:Float,angle:Int
	
	Local surf:TSurface=GetSurface(mesh,1)
	Local v:Int=0
	
	For angle = 1 To segments
		
		an1=angle*360.0/segments
		an2=angle*360.0/segments +360.0/segments
		an3=angle*360.0/segments +180.0/segments
		an4=angle*360.0/segments -180.0/segments

		VertexCoords surf, v + 0, radius1 * Cos(an1), radius1 * Sin(an1), 0
		VertexCoords surf, v + 1, radius1 * Cos(an2), radius1 * Sin(an2), 0
		VertexCoords surf, v + 2, radius2 * Cos(an3), radius2 * Sin(an3), 0
		VertexCoords surf, v + 3, radius2 * Cos(an4), radius2 * Sin(an4), 0
		
		v = v + 4
			
	Next

End Function



' ------------------------------------------------------------------------------------------------
' Creates a new System
' ------------------------------------------------------------------------------------------------
Function NewSystem(id:Int = 0)

	Local maxdist:Float = 0.0
				
	UpdateSystem(id)
	
	StarColor()

	For Local i:Int = 1 To PLANETS

		Local P:TPlanet = System.Planet[i]

		Local d:Float = P.DistanceAU
		If d > maxdist Then maxdist = d
					
	Next
	
	Local s:Double = (System.RadiusEstimatedKM / 10 ^ 8)
	ScaleEntity Star, s, s, s
	ScaleSprite StarSprite, s * 2, s * 2
	ScaleSprite StarGlow1, s * 1.5, s * 1.5
	ScaleSprite StarGlow2, s * 32, s * 32
	
	EntityAlpha StarSprite, 0.5
	EntityAlpha StarGlow1, 0.25
	EntityAlpha StarGlow2, 0.25

	Local d1:Float = System.HZMinMinimumPlanetDistanceAU
	Local d2:Float = System.HZMaxMinimumPlanetDistanceAU
	Local d4:Float = System.HZOptMinimumPlanetDistanceAU

	UpdateGlow(HotZone, 0, d2, SEGMENTS)
	UpdateGlow(HabZone, 0, d2, SEGMENTS)
	UpdateGlow(FroZone, 0, maxdist, SEGMENTS)
	UpdateGlow(OptZone, d1, d2, SEGMENTS)
	
	RotateMesh HotZone, 90, 0, 0
	RotateMesh HabZone, 90, 0, 0
	RotateMesh OptZone, 90, 0, 0
	RotateMesh FroZone, 90, 0, 0

End Function



' ------------------------------------------------------------------------------------------------
' Updates current Star System and Planet detail data
' ------------------------------------------------------------------------------------------------
Function UpdateSystem(id:Int = 0)

	System = New TSystem
		
	If id = 0 Then System.id = Rand(1000000000) Else System.id = id ' 517872798
	'System.id = 517872798
		
	System.Init()

	System.NewRandomSystem()

	'System.SpectralType = "G"
	'System.SubClass = 2
	'System.LuminanceClass = 5
	'System.Giant = 0
		
	System.CalcSystem()
		
	For Local i:Int = 1 To PLANETS
		
		System.Planet[i] = New TPlanet
		
		Local P:TPlanet = System.Planet[i]
		
		P.id = i
		P.DistanceAU = P.CalcDistanceAU(System, i)
						
		P.DistanceKM = P.DistanceAU * System.AstronomicalUnitKM / 1000000.0
		P.Mass = P.CalcMass(System, i)
		P.Density = Abs(Log10(P.Mass * 1.0 / 10.0 ^ 30.0)) / 1.25
		P.Volume = (P.Mass * 1000.0) / P.Density
		P.Radius = (Sqr(3.0 * P.Volume / (4.0 * Pi)) ^ 0.33333333) / 4.0
		P.EscapeVelocity = Sqr(System.GravitationalConstant * P.Mass / (P.Radius * 1000)) / 1000.0
		P.OrbitalSpeed = Sqr(System.GravitationalConstant * (System.Mass + P.Mass) / (P.DistanceKM * 1000000000000000.0))
		P.TidallyLockedTime = P.CalcTidallyLockedTime(System)
		P.TidallyLocked = P.IsTidallyLocked(System)
		P.SurfaceGravity = ((System.GravitationalConstant * (P.Mass / System.SunMass) * System.SunMass) / ((P.Radius * 1000) ^ 2)) / System.EarthGForce
		P.AtmoPressure = Sqr(P.SurfaceGravity) * Sqr(P.Density) ^ 0.5
		P.SidericDays = (P.DistanceAU * 365.25) + (Sqr(P.DistanceAU) * i ^ 2) * (1.0 / i)
		P.SidericYears = P.SidericDays / 365.25
		P.RotationSpeed = P.CalcRotationSpeed()
		
		P.AlbedoAlpha = Rnd(1) / 1.5
		P.AlbedoBeta = P.AlbedoAlpha / 2.0
		P.AlbedoTotal = P.AlbedoAlpha + P.AlbedoBeta
		P.SolarConstant = System.CorrectedHeatOutput / (4.0 * Pi * (P.DistanceKM * 1000000000.0) ^ 2)
		P.SolarCorrectedConstant = P.SolarConstant / 4.0
		P.SurfaceEnergy = P.SolarCorrectedConstant * (1 - P.AlbedoAlpha + P.AlbedoBeta)
				
		P.ElementMass = Sqr(P.Density * 25) * P.Density
			
		P.CalcAtmosphere()
		P.CalcHydrosphere()
		P.CalcLithosphere()

		P.GreenHouseEffect = P.CalcGreenHouseEffect(i)
		P.TemperatureKelvin = ((P.SurfaceEnergy * 1.0 / System.Boltzmann) ^ 0.25) + P.GreenHouseEffect
		P.TemperatureCelsius = P.TemperatureKelvin - System.CelsiusKelvin
		P.TemperatureMin = (P.TemperatureKelvin * Rnd(0.7, 0.9) * (1 + P.TidallyLocked))
		P.TemperatureMax = (P.TemperatureKelvin * Rnd(1.1, 1.3) * (1 + P.TidallyLocked))
		P.TemperatureMinCorrected = (P.TemperatureMin + P.GreenHouseEffect) - System.CelsiusKelvin
		P.TemperatureMaxCorrected = (P.TemperatureMax + P.GreenHouseEffect) - System.CelsiusKelvin
		P.Temperature = (P.TemperatureMinCorrected + P.TemperatureMaxCorrected) / 2.0
		P.TemperatureDifference = (P.TemperatureMaxCorrected - P.TemperatureMinCorrected) / 2 * (1 + P.TidallyLocked)
		
		P.CalcESI()
		
		P.CalcSizeCategory()
		P.CalcTemperatureCategory()
		P.CalcWeather()

		P.CalcCategory()

		P.Resources = Rnd(1) * Sqr(P.Density / i) / 2.5
		If P.Resources > 1 Then P.Resources = 1
		
		P.CalcEvaluationScore(System)

	Next
				
End Function

Function PlanetOutput(System:TSystem, i:Int)

	Local P:TPlanet = System.Planet[i]
	
	LogUpdate("")
	LogUpdate("-------------------------------------------------------------------------------")
	LogUpdate("Planet #" + i + " results")
	LogUpdate("-------------------------------------------------------------------------------")
	LogUpdate("")
	LogUpdate("Planet ID...................: " + P.id)
	LogUpdate("Distance....................: " + Round(P.DistanceKM, 0) + " Mio. KM (" + Round(P.DistanceAU, 1) + " AU)")
	LogUpdate("Mass........................: " + Round(P.Mass, 2) + " kg")
	LogUpdate("Density.....................: " + Round(P.Density, 2) + " g/cm3")
	LogUpdate("Volume......................: " + Round(P.Volume, 2) + " cm3")
	LogUpdate("Radius......................: " + Round(P.Radius, 0) + " km")
	LogUpdate("Tidal lock..................: " + YESNO[P.TidallyLocked] + " (after " + Round(P.TidallyLockedTime, 0) + " Years)")
	LogUpdate("Escape Velocity.............: " + Round(P.EscapeVelocity, 2) + " km/s")
	LogUpdate("Orbital Speed...............: " + Round(P.OrbitalSpeed, 2) + " km/s")
	LogUpdate("Surface Gravity.............: " + Round(P.SurfaceGravity, 2) + " g")
	LogUpdate("Atmospheric Pressure........: " + Round(P.AtmoPressure, 2) + " bar")
	LogUpdate("Sideric Period..............: " + Round(P.SidericDays, 2) + " Days")
	LogUpdate("Sideric Period..............: " + Round(P.SidericYears, 2) + " Years")
	LogUpdate("Rotation Speed..............: " + Round(P.RotationSpeed, 6) + " Days")
	LogUpdate("Planet Category.............: " + P.Category + " (" + P.TemperatureCategory + " " + P.SizeCategory + ")")
	LogUpdate("")
	LogUpdate("TEMPERATURE")
	LogUpdate("Albedo Alpha................: " + Round(P.AlbedoAlpha * 100, 2) + "%")
	LogUpdate("Albedo Beta.................: " + Round(P.AlbedoBeta * 100, 2) + "%")
	LogUpdate("Solar Constant..............: " + Round(P.SolarConstant, 2) + " Watt/s")
	LogUpdate("Solar Constant corrected....: " + Round(P.SolarCorrectedConstant, 2) + " Watt/s")
	LogUpdate("Surface Energy..............: " + Round(P.SurfaceEnergy, 2) + " Watt/s")
	LogUpdate("Greenhouse Effect...........: " + Round(P.GreenHouseEffect, 2) + " Kelvin")
	LogUpdate("Temperature Minimum.........: " + Round(P.TemperatureMinCorrected, 2) + " Degrees Celsius")
	LogUpdate("Temperature Average.........: " + Round(P.Temperature, 2) + " Degrees Celsius")
	LogUpdate("Temperature Maximum.........: " + Round(P.TemperatureMaxCorrected, 2) + " Degrees Celsius")
	LogUpdate("Temperature Difference......: " + Round(P.TemperatureDifference, 2) + " Degrees Celsius")
	LogUpdate("")
	LogUpdate("COMPOSITION")
	LogUpdate("Atmosphere mostly...........: " + P.Atmosphere + " (" + Round(P.MaximumAtmosphereMass, 2) + ")")
	LogUpdate("Hydrosphere mostly..........: " + P.Hydrosphere + " (" + Round(P.MaximumHydrosphereMass, 2) + ")")
	LogUpdate("Lithosphere heaviest Element: " + P.Lithosphere + " (" + Round(P.MaximumLithosphereMass, 2) + ")")
	LogUpdate("Global Weather situation....: " + P.Weather)
	LogUpdate("")
	LogUpdate("EVALUATION")
	LogUpdate("Resources Score.............: " + Round(P.Resources * 100, 0) + "%")
	LogUpdate("Evaluation Score............: " + Round(P.EvaluationScore, 0) + "%")
	LogUpdate("Earth Similarity Index (ESI): " + Round(P.ESI * 100, 2) + "%")
	LogUpdate("ESI Radius..................: " + Round(P.ESIRadius * 100, 2) + "%")
	LogUpdate("ESI Density.................: " + Round(P.ESIDensity * 100, 2) + "%")
	LogUpdate("ESI Temperature.............: " + Round(P.ESITemperature * 100, 2) + "%")
	LogUpdate("ESI Cosmic Speed............: " + Round(P.ESICosmicSpeed * 100, 2) + "%")
	LogUpdate("ESI Interior................: " + Round(P.ESIInterior * 100, 2) + "%")
	LogUpdate("ESI Surface.................: " + Round(P.ESISurface * 100, 2) + "%")


End Function

Function SystemOutput(System:TSystem)

	' Output
	LogUpdate("")
	LogUpdate("-------------------------------------------------------------------------------")
	LogUpdate("System results")
	LogUpdate("-------------------------------------------------------------------------------")
	LogUpdate("")
	LogUpdate("BASICS")
	LogUpdate("Star System ID..............: " + System.id)
	LogUpdate("Spectral Class..............: " + System.SpectralClass)
	LogUpdate("Star Position XYZ...........: " + Round(System.Position[0], 1) + " , " + Round(System.Position[1], 1) + " , " + Round(System.Position[2], 1))
	LogUpdate("Star Distance...............: " + Round(System.Distance, 2) + " LY")
	LogUpdate("")
	LogUpdate("SPECTRAL")
	LogUpdate("Absolute Magnitude..........: " + Round(System.AbsoluteMagnitude, 2) + " Mag")
	LogUpdate("Bolometric Magnitude........: " + Round(System.BolometricMagnitude, 2) + " Mag")
	LogUpdate("Temperature calculated......: " + Round(System.TemperatureCalculated, 0) + " Kelvin")
	LogUpdate("Temperature typical.........: " + Round(System.TemperatureTypical, 0) + " Kelvin")
	LogUpdate("Temperature estimated.......: " + Round(System.TemperatureEstimated, 0) + " Kelvin")
	LogUpdate("Luminance...................: " + Round(System.LuminanceWatt, 2) + " Watt/m2 (" + Round(System.Luminance, 6) + " x Sun)")
	LogUpdate("Infrared Radiation..........: " + Round(System.Infrared, 6) + " x Sun")
	LogUpdate("Peak Radiation at...........: " + Round(System.PeakRadiation, 0) + "nm (" + System.PeakVisible + ", " + System.SpectralLines + ")")
	LogUpdate("")
	LogUpdate("PHYSICS")
	LogUpdate("Radius estimated............: " + Round(System.RadiusEstimatedKM, 0) + " Kilometer (" + Round(System.RadiusEstimatedSun, 2) + " x Sun)")
	LogUpdate("Mass estimated..............: " + Round(System.Mass, 2) + " kg (" + Round(System.MassEstimated, 2) + " x Sun)")
	LogUpdate("Density.....................: " + Round(System.Density, 2) + " g/cm3 (" + Round(System.DensitySun, 4) + " x Sun)")
	LogUpdate("Surface Gravity.............: " + Round(System.Gravity, 2) + " m/s (" + Round(System.GravityG, 4) + "g)")
	LogUpdate("Escape Velocity at 0,1 AU...: " + Round(System.CalcEscapeVelocity(0.1), 2) + " m/s (" + Round(System.CalcEscapeVelocity(0.1) / System.EarthGForce, 2) + "g)")
	LogUpdate("Escape Velocity at 1,0 AU...: " + Round(System.CalcEscapeVelocity(1.0), 2) + " m/s (" + Round(System.CalcEscapeVelocity(1.0) / System.EarthGForce, 2) + "g)")
	LogUpdate("Escape Velocity at 10 AU....: " + Round(System.CalcEscapeVelocity(10.0), 2) + " m/s (" + Round(System.CalcEscapeVelocity(10.0) / System.EarthGForce, 2) + "g)")
	LogUpdate("Escape Velocity at 100 AU...: " + Round(System.CalcEscapeVelocity(100.0), 2) + " m/s (" + Round(System.CalcEscapeVelocity(100.0) / System.EarthGForce, 2) + "g)")
	LogUpdate("Escape Velocity at 1000 AU..: " + Round(System.CalcEscapeVelocity(1000.0), 2) + " m/s (" + Round(System.CalcEscapeVelocity(1000.0) / System.EarthGForce, 2) + "g)")
	LogUpdate("Escape Velocity at 0,1 LY...: " + Round(System.CalcEscapeVelocity(System.LightYearsAUs / 10.0), 2) + " m/s (" + Round(System.CalcEscapeVelocity(System.LightYearsAUs / 10.0) / System.EarthGForce, 2) + "g)")
	LogUpdate("Escape Velocity at 1 LY.....: " + Round(System.CalcEscapeVelocity(System.LightYearsAUs), 2) + " m/s (" + Round(System.CalcEscapeVelocity(System.LightYearsAUs) / System.EarthGForce, 2) + "g)")
	LogUpdate("")
	LogUpdate("LIFETIME")
	LogUpdate("Approximate Star Age........: " + Round(System.Age, 0) + " Years")
	LogUpdate("Maximum Lifetime............: " + Round(System.Lifetime, 0) + " Years")
	LogUpdate("Estimated Star Death in.....: " + Round(System.Death, 0) + " Years")
	LogUpdate("Star Lifetime passed........: " + Round(System.LifetimePassed, 2) + "%")
	LogUpdate("Star Fate...................: " + System.Fate)
	LogUpdate("")
	LogUpdate("LIFE SUPPORT")
	LogUpdate("Minimum Planet Distance.....: " + Round(System.MinimumPlanetDistance, 6) + " AU")
	LogUpdate("Minimum Habitable Zone......: " + Round(System.HZMinMinimumPlanetDistanceKM, 2) + " Mio. Kilometer (" + Round(System.HZMinMinimumPlanetDistanceAU, 6) + " AU)")
	LogUpdate("Optimum Habitable Zone......: " + Round(System.HZOptMinimumPlanetDistanceKM, 2) + " Mio. Kilometer (" + Round(System.HZOptMinimumPlanetDistanceAU, 6) + " AU)")
	LogUpdate("Maximum Habitable Zone......: " + Round(System.HZMaxMinimumPlanetDistanceKM, 2) + " Mio. Kilometer (" + Round(System.HZMaxMinimumPlanetDistanceAU, 6) + " AU)")
	LogUpdate("Metallicity.................: " + Round(System.Metallicity, 2) + " Fe/H")
	LogUpdate("Metallicity life suitable...: " + YESNO[System.MetallicitySuitable])
	LogUpdate("Star old enough for own life: " + YESNO[System.StarOldEnough])
	LogUpdate("Spectrum life suitable?.....: " + YESNO[System.SpectrumSuitable])
	
End Function