' ----------------------------------------------------------------------------
' Normalizes a value to a given range
' ----------------------------------------------------------------------------
Function Normalize:Float(value:Float = 128.0, vmin:Float = 0.0, vmax:Float = 255.0, nmin:Float = 0.0, nmax:Float = 1.0)
    
	Return ((value:Float - vmin:Float) / (vmax:Float - vmin:Float)) * (nmax:Float - nmin:Float) + nmin:Float
    
End Function

' ----------------------------------------------------------------------------
' Limit a INT value to a given range
' ----------------------------------------------------------------------------
Function LimitInt:Int(value:Int, mn:Int, mx:Int)
	
	If value>mx Then value=mx
	If value<mn Then value=mn
	
	Return value
	
End Function

' ----------------------------------------------------------------------------
' Limit a FLOAT value to a given range
' ----------------------------------------------------------------------------
Function LimitFloat:Float(value:Float, mn:Float, mx:Float)
	
	If value>mx Then value=mx
	If value<mn Then value=mn
	
	Return value
	
End Function

' ----------------------------------------------------------------------------
' Precisely rounds a FLOAT value to a INT value, different from INT()
' ----------------------------------------------------------------------------
Function RInt:Float(x:Float)

	If x >= 0 Then Return Floor(x) Else Return Ceil(x)
	
End Function

' ----------------------------------------------------------------------------
' 2D Distance Calculation
' ----------------------------------------------------------------------------
Function Distance2D:Int(x1:Int, z1:Int, x2:Int, z2:Int)
	
	Local x:Int = x1 - x2
	Local z:Int = z1 - z2
	
	Return Sqr(x * x + z * z)
	
End Function

' ----------------------------------------------------------------------------
' 3D Distance Calculation
' ----------------------------------------------------------------------------
Function Distance3D:Float(x1:Float, y1:Float, z1:Float, x2:Float, y2:Float, z2:Float)
	
	Local x:Float = x1 - x2
	Local y:Float = y1 - y2
	Local z:Float = z1 - z2
	
	Return Sqr(x * x + y * y + z * z)
	
End Function


Function Round:String(value:Double, floats:Int = 2, delimiter:Int = True)

	Local s:String=value
	Local e:Int=s.Find("e")
	Local exponent:Int

	Local i:Int=s.Find(".")

	Local integer:String
	Local floating:String

	' find out 10^exponent, positive and negative
	If e>0 Then exponent=Int(Mid(s,e+2,999)) Else exponent=Null

	' find out integer
	If i>0 Then

		integer=Mid(s,0,i+1)
		floating=Mid(s,i+2,floats)
		
		'If floats=0 Or Int(floating)=0 Then floating=Null
		If floats=0 Then floating=Null

	Else

		integer=s
		floating=Null
		
	EndIf

	' add dot dividers for integer
	If delimiter Then

		Local l:Int=Len(integer)
		Local out:String
		Local negative:Int

		If integer<0 Then negative=1

		For Local j:Int=l To 0 Step -1

			out:+Mid(integer, l - j, 1)

			If (j Mod 3) = 0 And j > 0 And (j < l - negative) Then out:+"."

		Next

		integer=out

	EndIf
	
	' output with exponent
	If floating And exponent And floating Then Return integer+","+floating+"x10^"+exponent
	If integer And exponent And (Not floating) Then Return integer+"x10^"+exponent

	' output without exponent
	If floating And (Not exponent) Then Return integer+","+floating
	If (Not floating) And (Not exponent) Then Return integer

EndFunction