' ------------------------------------------------------------------------------------------------
' Draw a 3D line using OpenGL functions:
' glEnable(GL_COLOR_MATERIAL) / glBegin(GL_LINES) [...] glEnd() / glDisable(GL_COLOR_MATERIAL)
' ------------------------------------------------------------------------------------------------
Function Line3D:TMesh(x0:Float, y0:Float, z0:Float, x1:Float, y1:Float, z1:Float, r:Int = 255, g:Int = 255, b:Int = 255, a:Float = 1.0)
		
	SetColor r, g, b
	SetAlpha(a)
		
	glVertex3f(x0, y0, -z0)
   	glVertex3f(x1, y1, -z1)
			
End Function



' ------------------------------------------------------------------------------------------------
' Rotate and scale a texture using separate calculations
' ------------------------------------------------------------------------------------------------
Function RoTex(texture:TTexture, angle:Float, Scale:Float = 1)

	Local x:Float
	Local y:Float

	ScaleTexture texture, Scale, Scale
	
	RotateTexture texture, angle
	
	x = Cos(angle) / Scale / 2.0
	y = Sin(angle) / Scale / 2.0
	
	PositionTexture texture, (x - 0.5) - y, (y - 0.5) + x
	
End Function



' ----------------------------------------------------------------------------
' Updates the size of a mesh around a sphere according to camera distance
' ----------------------------------------------------------------------------
Function UpdateScale:Float(cam:TCamera, target:TEntity = Null, Scale:Float = 1.0, radius:Float = 1.0)
	
	Local c1:Float, a1:Float, q1:Float, p1:Float, h1:Float, alpha1:Float, beta1:Float, gamma1:Float, alpha2:Float, b2:Float, c2:Float
		
	c1 = EntityDistance(cam, target)
	a1 = radius
	q1 = a1 * a1 / c1
	p1 = c1 - q1
	h1 = Sqr(p1 * q1)
	gamma1 = 90
	alpha1 = ATan(h1 / p1)
	beta1 = gamma1 - alpha1
	
	alpha2 = 90 - (90 - beta1)
	b2 = a1 / Tan(alpha2)
	c2 = (Sqr(a1 * a1 + b2 * b2)) / radius * Scale
	
	Return c2
	
End Function



' ----------------------------------------------------------------------------
' Pixel-Perfect Camera Sprites
' ----------------------------------------------------------------------------
Function PositionSprite(ent:TSprite, x:Float, y:Float, w:Float, h:Float, scaler:Float = 1.0)

	Local sw:Int = GraphicsWidth()
	Local sh:Int = GraphicsHeight()
	
	PositionEntity ent, x - (sw / 2.0), -(y - sh / 2.0) + 1.0, (sw / 2.0) * scaler
	
	ScaleSprite ent, w / 2.0 * scaler, h / 2.0 * scaler
	
End Function