' ------------------------------------------------------------------------------------------------
' Screen Globals
' ------------------------------------------------------------------------------------------------
Global SCREEN_WIDTH:Int = 512								' Graphics: Screen Width
Global SCREEN_HEIGHT:Int = 480	 							' Graphics: Screen Height
Global SCREEN_DEPTH:Int = 32								' Graphics: Screen Depth
Global SCREEN_MODE:Int = 2									' Graphics: Screen Mode
Global SCREEN_RATE:Int = 60									' Graphics: Screen Refresh Rate
Global SCREEN_DESKTOPWIDTH:Int = ClientWidth(Desktop())		' Graphics: Desktop Center horizontal
Global SCREEN_DESKTOPHEIGHT:Int = ClientHeight(Desktop())	' Graphics: Desktop Center vertical
Global SCREEN_CENTERX:Int = SCREEN_DESKTOPWIDTH / 2			' Graphics: Desktop Center horizontal
Global SCREEN_CENTERY:Int = SCREEN_DESKTOPHEIGHT / 2		' Graphics: Desktop Center vertical
Global SCREEN_TIMER:TTimer = CreateTimer(1000)				' Graphics: Maximum Refresh Rate

Global SCREEN_ASPECT:Float = SCREEN_WIDTH * 1.0 / SCREEN_HEIGHT
Global PROJECT_TITLE:String = "Star System Generator Demo by Krischan [www.extrasolar.de]"

' MAXGUI Variables
Global MAXGUI_Templates:String[] = ["Earth", "Moon", "Mars", "Jupiter"]
Global MAXGUI_Timer:TTimer = CreateTimer(60)
Global MAXGUI_LOGO:TPixmap = LoadPixmap("logo/logo.png")
Global MAXGUI_Resolutions:String[999]
Global MAXGUI_CURRENT_Width:Int
Global MAXGUI_CURRENT_Height:Int
Global MAXGUI_LAUNCH:Int = False

' MAXGUI Elements
Global MAXGUI_WINDOW:TGadget
Global MAXGUI_CANVAS:TGadget
Global MAXGUI_Graphics_Resolution:TGadget
Global MAXGUI_Graphics_Fullscreen:TGadget
Global MAXGUI_Graphics_Windowed:TGadget
Global MAXGUI_Template:TGadget
Global MAXGUI_Settings_Panel:TGadget
Global MAXGUI_Button_Launch:TGadget
Global MAXGUI_Button_Cancel:TGadget

Global DEMO_Process:TProcess


AppTitle = PROJECT_TITLE

' ------------------------------------------------------------------------------------------------
' Create MAXGUI Window
' ------------------------------------------------------------------------------------------------
MAXGUI_WINDOW = CreateWindow(PROJECT_TITLE, SCREEN_CENTERX - (SCREEN_WIDTH / 2), SCREEN_CENTERY - (SCREEN_HEIGHT / 2), SCREEN_WIDTH, SCREEN_HEIGHT, Null, WINDOW_TITLEBAR)

' ------------------------------------------------------------------------------------------------
' Create Graphics Canvas in Main Window
' ------------------------------------------------------------------------------------------------
MAXGUI_CANVAS = CreateCanvas(0, 0, SCREEN_WIDTH, MAXGUI_LOGO.Height, MAXGUI_WINDOW)
SetGadgetLayout MAXGUI_CANVAS, 1, 1, 1, 1
ActivateGadget(MAXGUI_CANVAS)

Local ys:Int = 256

' Create a nice panel around graphics section
MAXGUI_Settings_Panel = CreatePanel(10, ys + 10, SCREEN_WIDTH - 25, SCREEN_HEIGHT - 256 - 50, MAXGUI_WINDOW, PANEL_GROUP, "Demo Graphics settings")

' Create Resolution Selector Dropdown Box
CreateLabel("Graphics Resolution:", 96, 24 + 3, 128, 24, MAXGUI_Settings_Panel, Null)
MAXGUI_Graphics_Resolution = CreateComboBox(224, 24, 172, 32, MAXGUI_Settings_Panel)
GetResolutions()

' Create Template Dropdown Box
CreateLabel("System Seed:", 96, 48 + 3, 128, 24, MAXGUI_Settings_Panel, Null)
MAXGUI_Template = CreateComboBox(224, 48, 172, 32, MAXGUI_Settings_Panel)
For Local i:Int = 0 To 99

	AddGadgetItem(MAXGUI_Template, i)
	
Next
SelectGadgetItem(MAXGUI_Template, 1)

' Create Fullscreen / Windowed Radio Buttons
CreateLabel("Screen Mode:", 96, 72 + 3, 128, 24, MAXGUI_Settings_Panel, Null)
MAXGUI_Graphics_Fullscreen = CreateButton("Fullscreen", 224, 72, 96, 24, MAXGUI_Settings_Panel, BUTTON_RADIO)
MAXGUI_Graphics_Windowed = CreateButton("Windowed", 224 + 96, 72, 96, 24, MAXGUI_Settings_Panel, BUTTON_RADIO)
SetButtonState(MAXGUI_Graphics_Fullscreen, True)

' Launch Button
MAXGUI_Button_Launch = CreateButton("Launch Demo", 96 + 16, 110, 128, 24, MAXGUI_Settings_Panel)

' Cancel Button
MAXGUI_Button_Cancel = CreateButton("Cancel", 96 + 128 + 32, 110, 128, 24, MAXGUI_Settings_Panel)

' ------------------------------------------------------------------------------------------------
' Main loop
' ------------------------------------------------------------------------------------------------
Repeat

	' Wait for an Event
	Select WaitEvent()
		
		' Window has been closed
		Case EVENT_WINDOWCLOSE		
		
									' close all
									Appclose()

		' Application has been terminated
		Case EVENT_APPTERMINATE
		
									' close all
									Appclose()
							
		Case EVENT_GADGETACTION
									Select EventSource()
										
										Case MAXGUI_Button_Cancel
																	Appclose()
																	
										Case MAXGUI_Button_Launch
										
																	Local r1:String = MAXGUI_Resolutions[SelectedGadgetItem(MAXGUI_Graphics_Resolution)]
																	Local r2:Int = Instr(r1, "x", 1)
																	SYSTEM_SCREEN_WIDTH = Int(Mid(r1, 0, r2))
																	SYSTEM_SCREEN_HEIGHT = Int(Mid(r1, r2 + 1, Len(r1)))
																	SYSTEM_SCREEN_MODE = ButtonState(MAXGUI_Graphics_Fullscreen)
																	systemid = SelectedGadgetItem(MAXGUI_Template)
																	
																	'Notify(SYSTEM_SCREEN_WIDTH + "," + SYSTEM_SCREEN_HEIGHT + "," + SYSTEM_SCREEN_MODE + "," + systemid)
																	
																	FreeGadget MAXGUI_WINDOW
																	
																	MAXGUI_LAUNCH = True
										
									End Select

		Case EVENT_GADGETSELECT
		
		' Redraw Canvas
		Case EVENT_TIMERTICK

									Select EventSource()

										Case MAXGUI_Timer

											RedrawGadget MAXGUI_CANVAS

									End Select

		' 2D/3D GUI
		Case EVENT_GADGETPAINT

									' Set Graphics to Canvas
									SetGraphics CanvasGraphics(MAXGUI_CANVAS)

									Begin2D()
									
										DrawPixmap(MAXGUI_LOGO, SCREEN_WIDTH / 2 - MAXGUI_LOGO.width / 2, 0)

									End2D()

									Flip True
									
									

	EndSelect

	GCCollect()

Until MAXGUI_LAUNCH = True

' ------------------------------------------------------------------------------------------------
' Close the Application with a confirm Question
' ------------------------------------------------------------------------------------------------
Function Appclose()

	'If Confirm("Are you sure you want to exit?") Then

		EndGraphics
		End
		
	'Endif

End Function

' ------------------------------------------------------------------------------------------------
' Custom BeginMax2D function
' ------------------------------------------------------------------------------------------------
Function Begin2D()

	BeginMax2D()
	
	' hack to use drawtext in canvas
	glDisable(GL_CULL_FACE)
	glMatrixMode GL_PROJECTION
	glLoadIdentity
	glOrtho 0, GraphicsWidth(), GraphicsHeight(), 0, -1, 1

End Function



' ------------------------------------------------------------------------------------------------
' Custom EndMax2D function
' ------------------------------------------------------------------------------------------------
Function End2D()

	EndMax2D()

End Function

' ------------------------------------------------------------------------------------------------
' Retrieve and filter available Graphics Resolutions
' ------------------------------------------------------------------------------------------------
Function GetResolutions()

	Local w:Int
	Local h:Int
	Local d:Int
	Local z:Int

	Local m:Int
	Local i:Int
	
	Local add:Int
	Local index:Int = 0

	' iterate through all graphic modes
	For m = 0 To CountGraphicsModes() - 1
	
		' get available graphic modes and store them in variables
		GetGraphicsMode(m, w, h, d, z)
		
		' assuming we're adding them
		add = True
		
		' Resolution is already known or height too small? sort them out!
		For i = 0 To index
		
			If z <> 60 Then Add = False ; Exit
		
			If MAXGUI_Resolutions[i] = (w + "x" + h) Or h < 720 Then Add = False ; Exit
		
		Next
				
		' add all other resolutions to our Pulldown menu
		If add Then
		
			' add to Gadget
			AddGadgetItem(MAXGUI_Graphics_Resolution, w + "x" + h + " (" + d + "bit, " + z + "Hz)")
			
			' if the current Resolution matches Desktop resolution: preselect it!
			If w = DesktopWidth() And h = DesktopHeight() Then
			
				SelectGadgetItem(MAXGUI_Graphics_Resolution, index)
				MAXGUI_CURRENT_Width = w
				MAXGUI_CURRENT_Height = h
				
			EndIf
			
			' store the new Resolution in Array
			MAXGUI_Resolutions[index] = w + "x" + h
			
			' counter
			index:+1
			
			'Print "Found Resolution: " + w + "x" + h
			
		EndIf
				
	Next

End Function