Type TMedia

	Global media:TMap = CreateMap()
	
	Field desc:String
	Field cat:String
	Field obj:Object
	
	Field Loader_Text:TSprite
	Field Loader_Spiral:TSprite
	
	Field Loader_Text_Texture:TTexture = CreateTexture(32, 32)
	Field Loader_Spiral_Texture:TTexture = CreateTexture(32, 32)
	
	Field Loader_Progress:Int
	Field Loader_Scale:Float = 0.1

	
	' ------------------------------------------------------------------------------------------------
	' Init Media Loader
	' ------------------------------------------------------------------------------------------------
	Method Init(cam:TCamera)
	
		' Create a Sprite for the Loader Text
		Loader_Text = CreateSprite(cam)
		ScaleSprite Loader_Text, 2, 1
		PositionEntity Loader_Text, 0, 0.5, 5
		EntityBlend Loader_Text, 3
		EntityOrder Loader_Text, -2
		
		' Create a Sprite for the spinning Animation
		Loader_Spiral = CreateSprite(cam)
		ScaleSprite Loader_Spiral, 2, 2
		RotateEntity Loader_Spiral, 80, 0, 0
		SpriteViewMode Loader_Spiral, 2
		PositionEntity Loader_Spiral, 0, -0.5, 4
		EntityBlend Loader_Spiral, 3
		EntityColor Loader_Spiral, 0, 128, 255
		EntityOrder Loader_Spiral, -1
		
		' Hide both
		HideEntity Loader_Text
		HideEntity Loader_Spiral
	
	End Method

	
	' ------------------------------------------------------------------------------------------------
	' Update Media Loader
	' ------------------------------------------------------------------------------------------------
	Method Update()
	
		' Update Animation
		Loader_Progress:+5
		RoTex(Loader_Spiral_Texture, Loader_Progress)
		
		' Apply textures
		EntityTexture Loader_Text, Loader_Text_Texture
		EntityTexture Loader_Spiral, Loader_Spiral_Texture
		
		' Make visible
		ShowEntity Loader_Text
		ShowEntity Loader_Spiral
		
		' Render
		RenderWorld
		Flip False
		
		' Small pause
		Delay 1

		' Hide again
		HideEntity Loader_Text
		HideEntity Loader_Spiral

	End Method

		
	' ------------------------------------------------------------------------------------------------
	' Create a Media Object
	' ------------------------------------------------------------------------------------------------
	Method Create:TMedia(cat:String, desc:String, w:Int = 0, h:Int = 0, option:Int = Null)
	
		Local m:TMedia = New Tmedia

		m.desc = desc
		m.cat = cat
		
		Select cat
			
			Case "P" m.obj = CreatePixmap(w, h, option)
			Case "T" m.obj = CreateTexture(w, h, option)
			Case "X" m.obj = CreateSprite()
			Case "M" m.obj = CreateMesh()

		End Select
		
		LogUpdate("Media created: " + m.desc)
		
		MapInsert(MEDIA, desc, m)
		
		Return m
						
	End Method


	' ------------------------------------------------------------------------------------------------
	' Assign a Mesh Entity to a Media Object
	' ------------------------------------------------------------------------------------------------
	Method AssignMesh:TMedia(e:TMesh, desc:String)
	
		Local m:TMedia = New Tmedia
		m.desc = desc
		m.cat = "M"
		m.obj = e
		
		LogUpdate("Mesh assigned: " + m.desc)
		
		MapInsert(MEDIA, desc, m)
		
		Return m

	End Method


	' ------------------------------------------------------------------------------------------------
	' Assign a Light Entity to a Media Object
	' ------------------------------------------------------------------------------------------------
	Method AssignLight:TMedia(e:TLight, desc:String)
	
		Local m:TMedia = New Tmedia
		m.desc = desc
		m.cat = "L"
		m.obj = e
		
		LogUpdate("Light assigned: " + m.desc)
		
		MapInsert(MEDIA, desc, m)
		
		Return m

	End Method


	' ------------------------------------------------------------------------------------------------
	' Load a Media Object
	' ------------------------------------------------------------------------------------------------
	Method Load:TMedia(cat:String, desc:String, file:String, option:Int = Null, framewidth:Int = 0, frameheight:Int = 0, firstframe:Int = 1, framecount:Int = 1)
	
		Local m:TMedia = New Tmedia

		m.desc = desc
		m.cat = cat
		
		Update()
				
		Select cat
			
			Case "F" m.obj = LoadImageFont(file, option)
			Case "P" m.obj = LoadPixmap(file)
			Case "S" m.obj = LoadSound(file)
			Case "T" m.obj = LoadTexture(file, option)
			Case "A" m.obj = LoadAnimTexture(file, option, framewidth, frameheight, firstframe, framecount)
			Case "M" m.obj = LoadAnimMesh(file)
			Case "X" m.obj = LoadSprite(file)

		End Select
						
		MapInsert(MEDIA, desc, m)
		
		Return m
						
	End Method
	

	' ------------------------------------------------------------------------------------------------
	' Unloads a Media Object and frees the memory it used
	' ------------------------------------------------------------------------------------------------
	Method Unload(Key:String)
	
		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		Select m.cat
								
			Case "T" If TTexture(m.obj) Then FreeTexture TTexture(m.obj)
			Case "A" If TTexture(m.obj) Then FreeTexture TTexture(m.obj)
			Case "M" If TMesh(m.obj) Then FreeEntity TMesh(m.obj)
			Case "E" If TMesh(m.obj) Then FreeEntity TMesh(m.obj)

		End Select
		
		m.obj = Null
		m = Null
			
	End Method

					
	' ------------------------------------------------------------------------------------------------
	' Returns Texture Object
	' ------------------------------------------------------------------------------------------------
	Method GetT:TTexture(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		Update()
		
		If m.cat = "T" Or m.cat = "A" Then Return TTexture(m.obj)
		
		Return Null
	
	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Pixmap Object
	' ------------------------------------------------------------------------------------------------
	Method GetP:TPixmap(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m.cat = "P" Then Return TPixmap(m.obj)
		
		Return Null
	
	End Method
	

	' ------------------------------------------------------------------------------------------------
	' Returns Sound Object
	' ------------------------------------------------------------------------------------------------
	Method GetS:TSound(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m.cat = "S" Then Return TSound(m.obj)
		
		Return Null
	
	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Font Object
	' ------------------------------------------------------------------------------------------------
	Method GetF:TImageFont(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m.cat = "F" Then Return TImageFont(m.obj)
		
		Return Null
	
	End Method
	

	' ------------------------------------------------------------------------------------------------
	' Returns Mesh Object
	' ------------------------------------------------------------------------------------------------
	Method GetM:TMesh(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m.cat = "M" Then Return TMesh(m.obj)
		
		Return Null
	
	End Method

	' ------------------------------------------------------------------------------------------------
	' Returns Sprite Object
	' ------------------------------------------------------------------------------------------------
	Method GetX:TSprite(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m.cat = "X" Then Return TSprite(m.obj)
		
		Return Null
	
	End Method


	' ------------------------------------------------------------------------------------------------
	' Returns Light Object
	' ------------------------------------------------------------------------------------------------
	Method GetL:TLight(Key:String)

		Local m:TMedia = TMedia(MapValueForKey(MEDIA, Key))
		
		If m.cat = "L" Then Return TLight(m.obj)
		
		Return Null
	
	End Method
	
End Type