' ------------------------------------------------------------------------------------------------
' Milkyway Type
' ------------------------------------------------------------------------------------------------
Type TMilkyway

	Field masterpivot:TPivot

	' Variables
	Field Seed:Int = 42											' Base seed to recreate a scene
	Field Stars:Int = 25000										' Number of fixed stars
	Field MaxAngle:Float = 45									' Maximum angle of Star creation
	Field Scale:Float = 512										' Scale of Scene
	Field Height:Float = Scale * 0.75							' Height of Panorama
	Field Segments:Int = 32										' Panorama Mesh Segments
	Field StarScale:Float = 2.0									' Star size multiplicator
	Field FixedStars:Float = 1.0 / 25							' Amount of larger stars (0...1)
	
	' Colors
	Field RGB1:Int[] = [0, 0, 0, 0]								' Panorama upper vertex colors
	Field RGB2:Int[] = [224, 240, 255, 192]						' Panorama center vertex colors
	Field RGB3:Int[] = [0, 0, 0, 0]								' Panorama lower vertex colors
	
	' Detail
	Field DetailAlpha:Float = 1.0 / 2
	Field DetailRGB1:Int[] = [0, 0, 0, 0]						' Detail upper vertex colors
	Field DetailRGB2:Int[] = [255, 255, 255, 192]				' Detail center vertex colors
	Field DetailRGB3:Int[] = [0, 0, 0, 0]						' Detail upper vertex colors
	
	' Nebulae
	Field NebulaAlpha:Float = 1.0 / 2
	Field NebulaRows:Int = 4
	Field NebulaCols:Int = 4
	Field NebulaMinSize:Int = 96
	Field NebulaMedSize:Int = 192
	Field NebulaMaxSize:Int = 384
	Field NebulaMaxAngle:Float = 22.5
	Field NebulaAmount:Int = 250
	Field NebulaDensity:Float = 0.25							' Density of larger Nebulas

	' Entities
	Field Pivot:TPivot
	Field StarSphere:TMesh
	Field Panorama:TMesh
	Field Detail:TMesh
	Field Nebula:TMesh
	Field StarBox1:TMesh
	Field StarBox2:TMesh
	Field StarBox3:TMesh
	
	' Textures
	Field StarSphereTexture:TTexture
	Field PanoramaTexture:TTexture
	Field DetailTexture:TTexture
	Field NebulaTexture:TTexture
	Field Brightness:TTexture
	Field StarsTexture:TTexture
	Field StarBox1Texture:TTexture
	Field StarBox2Texture:TTexture
	Field StarBox3Texture:TTexture
	
	' Entity Order
	Field ORDER_StarSphere:Int = 1
	Field ORDER_Detail:Int = 2
	Field ORDER_Panorama:Int = 3
	Field ORDER_Nebula:Int = 4
	Field ORDER_StarBox1:Int = 5
	Field ORDER_StarBox2:Int = 6
	Field ORDER_StarBox3:Int = 7
	
	' Entity FX
	Field FX_StarSphere:Int = 1 + 2 + 8 + 32
	Field FX_Detail:Int = 1 + 2 + 8 + 32
	Field FX_Panorama:Int = 1 + 2 + 8 + 32
	Field FX_Nebula:Int = 1 + 2 + 8 + 32
	Field FX_StarBox1:Int = 1 + 2 + 8 + 32
	Field FX_StarBox2:Int = 1 + 2 + 8 + 32
	Field FX_StarBox3:Int = 1 + 2 + 8 + 32
	
	' Entity Blendings
	Field BLEND_StarSphere:Int = 1
	Field BLEND_Detail:Int = 3
	Field BLEND_Panorama:Int = 3
	Field BLEND_Nebula:Int = 3
	Field BLEND_StarBox1:Int = 3
	Field BLEND_StarBox2:Int = 3
	Field BLEND_StarBox3:Int = 3
	
	' Texture Blendings
	Field TEXBLEND_StarSphere:Int = 3
	Field TEXBLEND_Detail:Int = 5
	Field TEXBLEND_Panorama:Int = 5
	Field TEXBLEND_Nebula:Int = 5
	Field TEXBLEND_Stars:Int = 3
	Field TEXBLEND_StarBox1:Int = 2
	Field TEXBLEND_StarBox2:Int = 2
	Field TEXBLEND_StarBox3:Int = 3
		
	Field star:TQuad
	
	Field r:Int, g:Int, b:Int
	
	Field nebulalist:TList
	Field starlist:TList
		
	
	' --------------------------------------------------------------------------------------------
	' Initialize Milkyway Type
	' --------------------------------------------------------------------------------------------
	Method Init()
	
		SeedRnd Seed
					
		' Main Pivot
		Pivot = CreatePivot(masterpivot)
		
		nebulalist = CreateList()
		starlist = CreateList()
			
		' Create Meshes
		InitStarSphere(Pivot)									' Create Front Stars		
		InitPanorama(Pivot)										' Create Panorama
		InitDetail(Pivot)										' Create Detail
		InitNebula(Pivot)										' Create Nebulae
		InitStarBox1(Pivot)										' Create Background Stars #1
		InitStarBox2(Pivot)										' Create Background Stars #2
		InitStarBox3(Pivot)										' Create Background Stars #3
						
		' Reorder
		EntityOrder StarSphere, ORDER_StarSphere
		EntityOrder Panorama, ORDER_Panorama
		EntityOrder Nebula, ORDER_Nebula
		
		' Texture Scaling
		ScaleTexture NebulaTexture, NebulaRows, NebulaCols
			
		' Entity FX
		EntityFX StarSphere, FX_StarSphere
		EntityFX Panorama, FX_Panorama
		EntityFX Detail, FX_Detail
		EntityFX Nebula, FX_Nebula
		EntityFX StarBox1, FX_StarBox1
		EntityFX StarBox2, FX_StarBox2
		EntityFX StarBox3, FX_StarBox3
			
		' Entity Blending
		EntityBlend StarSphere, BLEND_StarSphere
		EntityBlend Panorama, BLEND_Panorama
		EntityBlend Detail, BLEND_Detail
		EntityBlend Nebula, BLEND_Nebula
		EntityBlend StarBox1, BLEND_StarBox1
		EntityBlend StarBox2, BLEND_StarBox2
		EntityBlend StarBox3, BLEND_StarBox3

		' Entity Texturing
		EntityTexture StarSphere, StarSphereTexture
		EntityTexture Panorama, PanoramaTexture
		EntityTexture Detail, StarsTexture, 0, 0
		EntityTexture Detail, DetailTexture, 0, 1
		EntityTexture Nebula, NebulaTexture
		EntityTexture StarBox1, StarBox1Texture
		EntityTexture StarBox2, StarBox2Texture
		EntityTexture StarBox3, StarBox3Texture
		
		ScaleTexture DetailTexture, 1.0 / 4, 1.0 / 1
		PositionTexture DetailTexture, Rnd(0, 1), Rnd(0, 1)

		ScaleTexture StarsTexture, 1.0 / 64, 1.0 / 16
		PositionTexture StarsTexture, Rnd(0, 1), Rnd(0, 1)
		
		ScaleTexture StarBox1Texture, 1.0 / 16, 1.0 / 16
		PositionTexture StarBox1Texture, Rnd(0, 1), Rnd(0, 1)
		
		ScaleTexture StarBox2Texture, 1.0 / 24, 1.0 / 24
		PositionTexture StarBox2Texture, Rnd(0, 1), Rnd(0, 1)
		
		ScaleTexture StarBox3Texture, 1.0 / 32, 1.0 / 32
		PositionTexture StarBox3Texture, Rnd(0, 1), Rnd(0, 1)
		
		' Texture Blending
		TextureBlend StarSphereTexture, TEXBLEND_StarSphere
		TextureBlend PanoramaTexture, TEXBLEND_Panorama
		TextureBlend DetailTexture, TEXBLEND_Detail
		TextureBlend DetailTexture, TEXBLEND_Stars
		TextureBlend NebulaTexture, TEXBLEND_Nebula
		TextureBlend StarBox1Texture, TEXBLEND_StarBox1
		TextureBlend StarBox2Texture, TEXBLEND_StarBox2
		TextureBlend StarBox3Texture, TEXBLEND_StarBox3
									
	End Method

		
	' --------------------------------------------------------------------------------------------
	' Remove Milkyway Type
	' --------------------------------------------------------------------------------------------
	Method Remove()
	
		If Panorama Then FreeEntity Panorama
		If StarSphere Then FreeEntity StarSphere
		If Detail Then FreeEntity Detail
		If Nebula Then FreeEntity Nebula		
		If Pivot Then FreeEntity pivot
						
	End Method


	' --------------------------------------------------------------------------------------------
	' Update Panorama Vertex Colors
	' --------------------------------------------------------------------------------------------
	Method UpdatePanoramaColor()
	
		Local surf:TSurface = GetSurface(Panorama, 1)
				
		For Local v:Int = 0 To CountVertices(surf) - 1 Step 6

			VertexColor surf, v + 0, RGB1[0], RGB1[1], RGB1[2], RGB1[3] / 255.0
			VertexColor surf, v + 1, RGB1[0], RGB1[1], RGB1[2], RGB1[3] / 255.0
			VertexColor surf, v + 2, RGB2[0], RGB2[1], RGB2[2], RGB2[3] / 255.0
			VertexColor surf, v + 3, RGB2[0], RGB2[1], RGB2[2], RGB2[3] / 255.0
			VertexColor surf, v + 4, RGB3[0], RGB3[1], RGB3[2], RGB3[3] / 255.0
			VertexColor surf, v + 5, RGB3[0], RGB3[1], RGB3[2], RGB3[3] / 255.0
		
		Next
			
	End Method
	
	
	
	' --------------------------------------------------------------------------------------------
	' Background Starbox #1
	' --------------------------------------------------------------------------------------------
	Method InitStarBox1(parent:TEntity = Null)
	
		Starbox1 = CreateCube(parent)
		FlipMesh Starbox1
		ScaleEntity Starbox1, Scale, Scale, Scale
			
	End Method

	
	
	' --------------------------------------------------------------------------------------------
	' Background Starbox #2
	' --------------------------------------------------------------------------------------------
	Method InitStarBox2(parent:TEntity = Null)
	
		Starbox2 = CreateCube(parent)
		FlipMesh Starbox2
		ScaleEntity Starbox2, Scale, Scale, Scale
			
	End Method

	
	
	' --------------------------------------------------------------------------------------------
	' Background Starbox #3
	' --------------------------------------------------------------------------------------------
	Method InitStarBox3(parent:TEntity = Null)
	
		Starbox3 = CreateCube(parent)
		FlipMesh Starbox3
		ScaleEntity Starbox3, Scale, Scale, Scale
			
	End Method
	
	
	
	' --------------------------------------------------------------------------------------------
	' Milkyway Panorama
	' --------------------------------------------------------------------------------------------
	Method InitPanorama(parent:TEntity = Null)
		
		Local a1:Float, a2:Float, inc:Float, x1:Float, z1:Float, x2:Float, z2:Float
		Local uu1:Float, uu2:Float, uv1:Float, uv2:Float, uv3:Float
		Local v0:Int, v1:Int, v2:Int, v3:Int, v4:Int, v5:Int
		Local surf:TSurface
			
		Panorama = CreateMesh(parent)
		surf = CreateSurface(Panorama)
			
		inc = 360.0 / Segments
			
		While a1 < 360
				
			a2 = (a1 + inc) Mod 360
					
			x1 = Scale * Cos(a1)
			z1 = Scale * Sin(a1)
			x2 = Scale * Cos(a2)
			z2 = Scale * Sin(a2)

			uu1 = 1.0 - (a1 / 360.0)
			uu2 = 1.0 - (a1 + inc) / 360.0

			uv1 = 1.00
			uv2 = 0.50
			uv3 = 0.00
			
			v0 = AddVertex(surf, x1, -Height, z1, uu1, uv1)
			v1 = AddVertex(surf, x2, -Height, z2, uu2, uv1)
			v2 = AddVertex(surf, x1, 0, z1, uu1, uv2)
			v3 = AddVertex(surf, x2, 0, z2, uu2, uv2)
			v4 = AddVertex(surf, x1, Height, z1, uu1, uv3)
			v5 = AddVertex(surf, x2, Height, z2, uu2, uv3)
			
			VertexColor surf, v0, RGB1[0], RGB1[1], RGB1[2], RGB1[3] / 255.0
			VertexColor surf, v1, RGB1[0], RGB1[1], RGB1[2], RGB1[3] / 255.0
			VertexColor surf, v2, RGB2[0], RGB2[1], RGB2[2], RGB2[3] / 255.0
			VertexColor surf, v3, RGB2[0], RGB2[1], RGB2[2], RGB2[3] / 255.0
			VertexColor surf, v4, RGB3[0], RGB3[1], RGB3[2], RGB3[3] / 255.0
			VertexColor surf, v5, RGB3[0], RGB3[1], RGB3[2], RGB3[3] / 255.0
			
			AddTriangle surf, v0, v1, v3
			AddTriangle surf, v3, v2, v0
			AddTriangle surf, v2, v5, v4
			AddTriangle surf, v5, v2, v3
			
			a1:+inc
					
		Wend
				
	End Method

	
	
	' --------------------------------------------------------------------------------------------
	' Detail Panorama
	' --------------------------------------------------------------------------------------------
	Method InitDetail(parent:TEntity = Null)
		
		Local a1:Float, a2:Float, inc:Float, x1:Float, z1:Float, x2:Float, z2:Float
		Local uu1:Float, uu2:Float, uv1:Float, uv2:Float, uv3:Float
		Local v0:Int, v1:Int, v2:Int, v3:Int, v4:Int, v5:Int
		Local surf:TSurface
			
		Detail = CreateMesh(parent)
		surf = CreateSurface(Detail)
			
		inc = 360.0 / Segments
			
		While a1 < 360
				
			a2 = (a1 + inc) Mod 360
					
			x1 = Scale * Cos(a1)
			z1 = Scale * Sin(a1)
			x2 = Scale * Cos(a2)
			z2 = Scale * Sin(a2)

			uu1 = 1.0 - (a1 / 360.0)
			uu2 = 1.0 - (a1 + inc) / 360.0

			uv1 = 1.00
			uv2 = 0.50
			uv3 = 0.00
			
			v0 = AddVertex(surf, x1, -Height, z1, uu1, uv1)
			v1 = AddVertex(surf, x2, -Height, z2, uu2, uv1)
			v2 = AddVertex(surf, x1, 0, z1, uu1, uv2)
			v3 = AddVertex(surf, x2, 0, z2, uu2, uv2)
			v4 = AddVertex(surf, x1, Height, z1, uu1, uv3)
			v5 = AddVertex(surf, x2, Height, z2, uu2, uv3)
			
			VertexColor surf, v0, DetailRGB1[0], DetailRGB1[1], DetailRGB1[2], DetailRGB1[3] / 255.0 * DetailAlpha
			VertexColor surf, v1, DetailRGB1[0], DetailRGB1[1], DetailRGB1[2], DetailRGB1[3] / 255.0 * DetailAlpha
			VertexColor surf, v2, DetailRGB2[0], DetailRGB2[1], DetailRGB2[2], DetailRGB2[3] / 255.0 * DetailAlpha
			VertexColor surf, v3, DetailRGB2[0], DetailRGB2[1], DetailRGB2[2], DetailRGB2[3] / 255.0 * DetailAlpha
			VertexColor surf, v4, DetailRGB3[0], DetailRGB3[1], DetailRGB3[2], DetailRGB3[3] / 255.0 * DetailAlpha
			VertexColor surf, v5, DetailRGB3[0], DetailRGB3[1], DetailRGB3[2], DetailRGB3[3] / 255.0 * DetailAlpha
			
			AddTriangle surf, v0, v1, v3
			AddTriangle surf, v3, v2, v0
			AddTriangle surf, v2, v5, v4
			AddTriangle surf, v5, v2, v3
			
			a1:+inc
					
		Wend
				
	End Method	
	
	
	
	' --------------------------------------------------------------------------------------------
	' Fixed Stars
	' --------------------------------------------------------------------------------------------
	Method InitStarSphere:TMesh(parent:TEntity = Null)
		
		Local i:Int
		Local surf:TSurface
		Local col:Float
		Local angle:Float
		
		StarSphere = CreateMesh(parent)
		surf = CreateSurface(StarSphere)
						
		For i = 1 To Stars
		
			star = New TQuad
			star.mesh = StarSphere
			star.surf = surf
			star.scalex = Rnd(Rnd(0.25, 0.35), Rnd(0.35, 0.5)) * StarScale
			angle = 60
			
			If Rnd(1) > (1 - FixedStars) Then
			
				star.scalex = Rnd(Rnd(0.5, 1.0), Rnd(1.0, Rnd(1.25, Rnd(1.5, 2)))) * StarScale
				angle = MaxAngle
			
			EndIf
			
			star.scaley = star.scalex
			
			' color
			col = Rnd(1)
			
			' orange stars
			If col > 0.90 And col <= 1.00 Then
			
				r = 255
				g = 0
				b = 0
				star.Scale = Rnd(0.25, 0.5)
				
			EndIf
			
			' yellow stars
			If col > 0.80 And col <= 0.90 Then
			
				r = 255
				g = 255
				b = 0
				star.Scale = Rnd(0.25, 0.75)
				
			EndIf
			
			' blue stars	
			If col > 0.5 And col <= 0.80 Then
			
				r = 0
				g = 0
				b = 255
				star.Scale = Rnd(0.75, 1.25)
				
			EndIf
			
			' white stars
			If col > 0.00 And col <= 0.5 Then
			
				r = 255
				g = 255
				b = 255
				star.Scale = Rnd(0.65, 0.85)
				
			EndIf
			
			'r = 255
			'g = 255
			'b = 255
				
			star.RGB1 = [r, g, b]
			star.RGB2 = [r, g, b]
			star.RGB3 = [r, g, b]
			star.RGB4 = [r, g, b]
						
			star.x = Rnd(-100, 100)
			star.y = Rnd(-angle, angle)
			star.z = Rnd(-100, 100)
					
			TFormNormal star.x, star.y, star.z, Null, Null
			
			Local d:Float = Rnd(1.0, 1.5)
			
			star.x = TFormedX() * Scale * d
			star.y = TFormedY() * Scale * d
			star.z = TFormedZ() * Scale * d
				
			star.add()
			star.AlignToTarget(star.x, star.y, star.z)
			ListAddLast(starlist, Star)
			
		Next
		
	End Method
	
		
	' --------------------------------------------------------------------------------------------
	' Detail Nebula
	' --------------------------------------------------------------------------------------------
	Method InitNebula(parent:TEntity = Null)
	
		Local i:Int
		Local q:TQuad
		Local surf:TSurface
		
		Nebula = CreateMesh(parent)
		surf = CreateSurface(Nebula)
						
		For i = 1 To NebulaAmount
		
			q = New TQuad
			q.mesh = Nebula
			q.surf = surf
			q.scalex = Rnd(NebulaMinSize, NebulaMedSize)
			q.scaley = q.scalex * Rnd(0.75, 1.25)
			
			RandomNebulaColor(False)
			q.RGB1 = [r, g, b]
			
			RandomNebulaColor(False)
			q.RGB2 = [r, g, b]
			
			RandomNebulaColor(False)
			q.RGB3 = [r, g, b]
			
			RandomNebulaColor(False)
			q.RGB4 = [r, g, b]
						
			q.Alpha1 = Rnd(1) * NebulaAlpha
			q.Alpha2 = Rnd(1) * NebulaAlpha
			q.Alpha3 = Rnd(1) * NebulaAlpha
			q.Alpha4 = Rnd(1) * NebulaAlpha
			
			Local num:Int = Rand(0, (NebulaRows * NebulaCols) - 1)
			Local row:Int = Int(num) / NebulaRows
			Local col:Int = Int(num) - (row * NebulaCols)
			
			q.x = Rnd(-100, 100)
			q.y = Rnd(-NebulaMaxAngle, NebulaMaxAngle)
			q.z = Rnd(-100, 100)
			
			If Rnd(1) > (1 - NebulaDensity) Then
				
				q.y = Rnd(-100, 100)
				q.scalex = Rnd(NebulaMedSize, NebulaMaxSize)
				q.scaley = q.scalex
		
				RandomNebulaColor()
				q.RGB1 = [r, g, b]
				
				RandomNebulaColor()
				q.RGB2 = [r, g, b]
				
				RandomNebulaColor()
				q.RGB3 = [r, g, b]
				
				RandomNebulaColor()
				q.RGB4 = [r, g, b]
								
			End If
			
			TFormNormal q.x, q.y, q.z, Null, Null
			
			Local d:Float = Rnd(1.0, 1.5)
						
			q.x = TFormedX() * Scale * d
			q.y = TFormedY() * Scale * d
			q.z = TFormedZ() * Scale * d
				
			q.Add(row, col)
			q.AlignToTarget(q.x, q.y, q.z, Rnd(360))
			ListAddLast(nebulalist, q)
			
		Next	
	
	End Method
	
	' --------------------------------------------------------------------------------------------
	' Create realistic random nebula colors (red, blue, white) plus darken/brighten
	' --------------------------------------------------------------------------------------------
	Method RandomNebulaColor(Darken:Int = True, brighten:Int = True)
			
		Local col:Float = Rnd(0, 1)
		Local c:Int = Rand(0, 255)

		If col > 0.0 Then r = 0 ; g = c / 2 ; b = c
		'If col > 0.75 Then r = c ; g = c / 16; b = 0
		
		If Darken And Rnd(0, 1) > 0.5 Then r = 0;g = 0;b = 0
		
		If brighten And Rnd(0, 1) > 0.75 Then r = c;g = c;b = c
		
		If brighten And Rnd(0, 1) > 0.95 Then r = 255; g = 255; b = 255
				
	
	End Method

	
	' --------------------------------------------------------------------------------------------
	' Hold position
	' --------------------------------------------------------------------------------------------
	Method KeepPosition(parent:TEntity)

		PositionEntity Pivot, EntityX(parent, 1), EntityY(parent, 1), EntityZ(parent, 1)

	End Method


End Type