' --------------------------------------------------------------------------------
' Simple digit formatting output of a number
' --------------------------------------------------------------------------------
Function DigOut:String(in:String, delimiter:String = ".")

	Local l:Int = Len(in)
	Local j:Int
	Local negative:Int
	Local out:String
	
	If Int(in) < 0 Then negative = 1
	
	For j = l To 0 Step - 1

		out:+Mid(in, l - j, 1)

		If (j Mod 3) = 0 And j > 0 And (j < l - negative) Then out:+"."

	Next
	
	Return out

End Function



' --------------------------------------------------------------------------------
' Simple leading spaces for nice number formatting output
' --------------------------------------------------------------------------------
Function FormOut:String(in:String, spaces:Int = 2, prefix:String = " ")

	Local i:Int
	Local l:Int = Len(in)
	Local out:String
		
	If l < spaces Then
	
		For i = 0 To spaces - l - 1
		
			out:+prefix
		
		Next
	
	End If
	
	Return out + in
	
End Function




' --------------------------------------------------------------------------------
' Simple spacing for nice string formatting output
' --------------------------------------------------------------------------------
Function FormStr:String(in:String, spaces:Int = 2, suffix:String = " ")

	Local i:Int
	Local l:Int = Len(in)
	Local out:String
		
	If l < spaces Then
	
		For i = 0 To spaces - l - 1
		
			out:+suffix
		
		Next
	
	End If
	
	Return in + out
	
End Function



' ------------------------------------------------------------------------------------------------
' Join an array to a string
' ------------------------------------------------------------------------------------------------
Function Join:String(arr:String[])

	Local r:String

	For Local i:Int = 0 To arr.length-1
		
		r:+arr[i]
		
	Next
	
	Return r

End Function



' ------------------------------------------------------------------------------------------------
' Drawtext with wordwrap function
' ------------------------------------------------------------------------------------------------
Const H_ALIGN_LEFT:Int = 0
Const H_ALIGN_CENTER:Int = 1
Const H_ALIGN_RIGHT:Int = 2
Const V_ALIGN_TOP:Int = 0
Const V_ALIGN_MIDDLE:Int = 4
Const V_ALIGN_BOTTOM:Int = 8

Function DrawTextEx(t:String, x:Float, y:Float, w:Float = 0, h:Float = 0, align:Int = 0)

	Local textlines:String[1]
	Local linecount:Int = 1
	textlines[0] = t:String

	Local fitwidth:Int = True
	
	If w <= 0 Then
	
		' No fixed limit, if w isn't set
		w = 100000
		fitwidth = False
		
	End If
				
	linecount = 1
	textlines[0] = ""
	Local index:Int = 0
	Local oldindex:Int = 0
	Local newline:Int = False
	
	While oldindex < t.length - 1
		
		newline = False
			
		index = t.length - 1
			
		For Local i:Int = oldindex To t.length - 1
		
			If t[i] = 10 Then
			
				index = i
				newline = True
				Exit
				
			ElseIf t[i] <= 32 Then
			
				index = i
				Exit
				
			EndIf
			
		Next
		
		If newline
		
			If TextWidth(textlines[linecount - 1] + t[oldindex..index + 1]) < w Then
			
				textlines[linecount - 1]:+t[oldindex..index + 1]
				linecount:+1
				textlines = textlines[..linecount]
				textlines[linecount - 1] = ""
				
			Else
			
				linecount:+1
				textlines = textlines[..linecount]
				textlines[linecount - 1] = " " + t[oldindex..index + 1]
				
			EndIf
			
		ElseIf TextWidth(textlines[linecount - 1] + t[oldindex..index + 1]) < w Or textlines[linecount - 1].length = 0 Then
		
			textlines[linecount - 1]:+t[oldindex..index + 1]
			
		Else
		
			linecount:+1
			textlines = textlines[..linecount]
			textlines[linecount - 1] = ""
			index = oldindex - 1
			
		EndIf
		
		oldindex = index + 1
		
	Wend

	If fitwidth = False Then
	
		' Reset w to 0
		w = 0
		
	EndIf
				
	If align & V_ALIGN_MIDDLE Then
	
		y = y + h * 0.5 - TextHeight(t:String) * linecount *.5
		
	ElseIf align & V_ALIGN_BOTTOM Then
	
		y = y + h - TextHeight(t:String) * linecount
		
	EndIf

	Local dx:Float

	For t:String = EachIn textlines
	
		dx = x
		
		If align & H_ALIGN_CENTER Then
		
			dx = x + w * 0.5 - TextWidth(t:String) *.5
			
		ElseIf align & H_ALIGN_RIGHT
		
			dx = x + w - TextWidth(t:String)
			
		EndIf

		DrawText t, dx, y

		y:+TextHeight(t:String)
		
	Next
	
End Function


Function UTF:String(unicode:String)

	Local i:Int
	Local char:String
	Local bytes:String
	Local byte1:String
	Local byte2:String
	Local byte3:String
	Local result:String = ""
		
	For i = 1 To Len(unicode)
		
		char = Mid:String(Unicode, i, 1)
		
		If Asc(char) < 128 Then
		
			RESULT:+char
			
		ElseIf Asc(char) > 127 And Asc(char) < 2048 Then
		
			bytes = Right(Bin(Asc(char)), 11)

			byte1 = "110" + Left(bytes, 5)
			result:+Chr(Bin2Int(byte1))

			byte2 = "10" + Right(bytes, 6)
			result:+Chr(Bin2Int(byte2))
			
		Else
		
			bytes = Right(Bin(Asc(char)), 16)

			byte1 = "1110" + Left(bytes, 4)
			result:+Chr(Bin2Int(byte1))

			byte2 = "10" + Mid(bytes, 5, 6)
			result:+Chr(Bin2Int(byte2))

			Byte3 = "10" + Right(bytes, 6)
			result:+Chr(Bin2Int(Byte3))
					
		EndIf
		
	Next
	
	Return RESULT
	
EndFunction

Function Unicode:String(UTF_8:String)

	Local i:Int
	Local RESULT:String = ""
	Local UTF:String = ""
	Local char:String
	Local b:String
	Local length:Int = 0
	Local last:String = ""
	
	For i = 1 To Len(UTF_8)
	
		char = Mid(UTF_8, i, 1)
		
		B = Right(Bin(Asc(Char)), 8)
		
		If length > 0 Then
		
			UTF:+Right(B, 6)
			
			length:-1
			
			If Left(B, 2) <> "10" Then
			
				Length=0
				RESULT:+last
				last = ""
				
			ElseIf length = 0 Then
			
				RESULT:+Chr(Bin2Int(UTF))
				
			EndIf
			
		EndIf
			
		If length = 0 Then
		
			If Left(B, 1) = "0" Then
			
				Result:+Char
				length = 0
								
			ElseIf Left(B, 3) = "110" Then
			
				last = CHAR
				UTF = Right(B, 5)
				length = 1
				
			ElseIf Left(B, 4) = "1110" Then
			
				UTF = Right(B, 5)
				length = 2
				
			ElseIf Left:String(B, 4) = "11110"
			
				UTF = Right(B, 5)
				length = 3
				
			EndIf
			
		EndIf
		
	Next
	
	Return RESULT
	
EndFunction

Function Bin2Int:Int(binary:String)

	Return Int("%" + binary)
	
EndFunction